package com.develop.main.ui

import android.content.Intent
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import androidx.databinding.ViewDataBinding
import androidx.drawerlayout.widget.DrawerLayout.DrawerListener
import com.alibaba.android.arouter.facade.annotation.Route
import com.develop.base.ext.navigateTo
import com.develop.base.ext.resId2Dimension
import com.develop.base.manager.WifiHelp
import com.develop.base.widgets.CommonPopupWindow
import com.develop.common.data_repo.db.DataFactory
import com.develop.common.data_repo.db.FunType
import com.develop.common.data_repo.db.RecipesType
import com.develop.common.router.Screens
import com.develop.common.tag.*
import com.develop.common.ui.CommonBVMActivity
import com.develop.common.widget.HomeFunctionTopBar
import com.develop.common.widget.NewsLayout
import com.develop.main.R
import com.develop.main.databinding.ActivityHomeBinding
import com.develop.main.viewmodel.HomeViewModel

@Route(path = Screens.Main.HOME)
class HomeActivity : CommonBVMActivity<ActivityHomeBinding, HomeViewModel>() {

    private val modesFragment by lazy {
        ModesFragment()
    }

    private val recipesFragment by lazy {
        RecipesFragment()
    }

    private val newsPopupWindow by lazy {
        CommonPopupWindow.ViewBuilder<NewsLayout>().width(LinearLayout.LayoutParams.MATCH_PARENT)
            .height(dp1080 + dp160).outsideTouchable(true).focusable(true).alpha(0.5f)
            .clippingEnabled(false).view(NewsLayout(this)).onShowBefore { popupWindow, view ->
                view.apply {
                    updateNewsData(DataFactory.genTestNewsList())
                    onNewsBackClickListener = object : NewsLayout.OnNewsNewsBackClickListener {
                        override fun onNewsBackClick() {
                            popupWindow.dismiss()
                        }
                    }
                }
            }.build<ViewDataBinding>(this)
    }

    private var currModeEntrance = Recipes

    private lateinit var wifiHelp: WifiHelp

    override fun createViewModel(): HomeViewModel {
        return getViewModel(HomeViewModel::class.java)
    }

    override fun createViewBinding(inflater: LayoutInflater): ActivityHomeBinding {
        return ActivityHomeBinding.inflate(inflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initFragment(intent)
        initCallbackAndListener()
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        initFragment(intent, true)
    }

    private fun initFragment(intent: Intent?, isFromNewIntent: Boolean = false) {
        currModeEntrance = intent?.extras?.getInt(MODE_ENTRANCE) ?: Recipes
        if (currModeEntrance == Recipes) {
            binding.functionTopBar.setCurrentFunType(FunType.LocalRecipes)
            if (isFromNewIntent) {
                replaceFragment(R.id.fragment_container, recipesFragment)
            } else {
                addFragment(R.id.fragment_container, recipesFragment)
            }
        } else {
            binding.functionTopBar.setCurrentFunType(FunType.Modes)
            if (isFromNewIntent) {
                replaceFragment(R.id.fragment_container, modesFragment)
            } else {
                addFragment(R.id.fragment_container, modesFragment)
            }
        }
    }

    private fun initCallbackAndListener() {
        wifiHelp = WifiHelp.Build(this).setAlreadyConnectionCallback {
            it?.apply {
                binding.functionTopBar.setNetRssi(rssi)

            }
        }.build()
        binding.functionTopBar.onHomeFunctionClickListener =
            object : HomeFunctionTopBar.OnHomeFunctionClickListener {
                override fun onModeClick(funType: FunType) {
                    if (funType == FunType.LocalRecipes) {
                        replaceFragment(R.id.fragment_container, recipesFragment)
                        viewModel.recipesType = RecipesType.LOCAL
                        viewModel.refreshRecipesLiveData.postValue(true)
                    } else if (funType == FunType.Modes) {
                        replaceFragment(R.id.fragment_container, modesFragment)
                    }
                }

                override fun onNotifyClick() {
                    newsPopupWindow.showAtLocation(binding.functionTopBar, Gravity.TOP, 0, 0)

                }

                override fun onTabClick() {
                    binding.contentView.openDrawer(binding.rightContentLayout)
                }

                override fun onWifiClick() {
                    navigateTo(Screens.Setting.WIFI) {
                        val bundle = Bundle()
                        bundle.putBoolean(WIFI_FROM_MAIN_SETTING, true)
                        with(bundle)
                    }
                }

                override fun onUserClick() {
                    navigateTo(Screens.Auth.MEMBER) {
                        withBoolean("enter_from_home", true)
                    }
                }

            }

        binding.rightContentLayout.setOnClickListener {

        }

        binding.settingLayout.setOnClickListener {
            navigateTo(Screens.Setting.MAIN_SETTING)
            binding.contentView.closeDrawers()
        }

        binding.favouriteLayout.setOnClickListener {
            val bundle = Bundle()
            bundle.putString(
                FOOL_LIST_TITLE,
                getString(com.develop.common.R.string.favourite_recipes)
            )
            bundle.putInt(FOOD_LIST_TYPE, Favourite)
            navigateTo(Screens.Cook.FOOD_LIST) {
                with(bundle)
            }
            binding.contentView.closeDrawers()
        }

        binding.historyLayout.setOnClickListener {
            val bundle = Bundle()
            bundle.putString(FOOL_LIST_TITLE, getString(com.develop.common.R.string.cook_history))
            bundle.putInt(FOOD_LIST_TYPE, History)
            navigateTo(Screens.Cook.FOOD_LIST) {
                with(bundle)
            }
            binding.contentView.closeDrawers()
        }

        binding.policyLayout.setOnClickListener {

            val bundle = Bundle()
            bundle.putBoolean(POLICY_FROM_MAIN_SETTING, true)
            navigateTo(Screens.Setting.POLICY) {
                with(bundle)
            }
            binding.contentView.closeDrawers()
        }

        viewModel.apply {
            netFoodLiveData.observe(this@HomeActivity) {
                binding.functionTopBar.updateNetFunType()
            }
        }

    }

    companion object {
        var dp1080 = (com.develop.base.R.dimen.convert_1080px).resId2Dimension().toInt()
        var dp160 = (com.develop.base.R.dimen.convert_160px).resId2Dimension().toInt()
    }

    override fun onResume() {
        super.onResume()
        wifiHelp.scan()
    }

    override fun onDestroy() {
        super.onDestroy()
        wifiHelp.destroy()
    }
}